﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;

namespace ORM03
{
    class EntityHelper
    {
        public const string connectionString = "";

        public static int Insert<T>(T entity)
        {
            var type = typeof(T);
            Dictionary<string, object> parameters = new Dictionary<string, object>();
            var properties = type.GetProperties();

            string tableName = string.Empty;
            TableAttribute[] tableAttrs = (TableAttribute[])type.GetCustomAttributes(typeof(TableAttribute), true);
            if (tableAttrs.Length > 0)
            {
                tableName = tableAttrs[0].Name;
            }
            else
            {
                tableName = type.Name;
            }

            /*将所有的列放到集合里*/
            List<ColumnAttribute> columns = new List<ColumnAttribute>();
            for (int i = 0; i < properties.Length; i++)
            {
                var pi = properties[i];
                var attrs = (ColumnAttribute[])pi.GetCustomAttributes(typeof(ColumnAttribute), true);
                if (attrs.Length > 0)
                {
                    columns.Add(attrs[0]);
                }
            }


            StringBuilder sql = new StringBuilder();
            sql.Append("INSERT INTO [").Append(tableName).Append("](");
            int paramIndex = 0;

            for (int i = 0; i < properties.Length; i++)
            {
                var pi = properties[i];
                var attrs = (ColumnAttribute[])pi.GetCustomAttributes(typeof(ColumnAttribute), true);
                if (attrs.Length > 0 && attrs[0].IsGenerated == false)
                {
                    if (paramIndex > 0)
                        sql.Append(",");

                    sql.Append(attrs[0].Name);
                    paramIndex++;
                }
            }

            sql.Append(") VALUES (");
            paramIndex = 0;
            for (int i = 0; i < properties.Length; i++)
            {
                var pi = properties[i];
                var attrs = (ColumnAttribute[])pi.GetCustomAttributes(typeof(ColumnAttribute), true);
                if (attrs.Length > 0 && attrs[0].IsGenerated == false)
                {
                    if (paramIndex > 0)
                        sql.Append(",");

                    sql.Append("@p").Append(paramIndex);
                    parameters.Add("@p" + paramIndex, pi.GetValue(entity, null));
                    paramIndex++;
                }
            }

            sql.Append(")");
            Console.WriteLine(sql);

            SqlConnection conn = new SqlConnection(connectionString);
            var cmd = conn.CreateCommand();
            cmd.CommandText = sql.ToString();
            foreach (var item in parameters)
            {
                var pa = cmd.CreateParameter();
                pa.ParameterName = item.Key;
                pa.Value = item.Value ?? DBNull.Value;
                cmd.Parameters.Add(pa);
            }


            conn.Open();
            return cmd.ExecuteNonQuery();
        }

        public static int Update<T>(T entity)
        {
            var type = typeof(T);
            Dictionary<string, object> parameters = new Dictionary<string, object>();
            var properties = type.GetProperties();

            string tableName = string.Empty;
            TableAttribute[] tableAttrs = (TableAttribute[])type.GetCustomAttributes(typeof(TableAttribute), true);
            if (tableAttrs.Length > 0)
            {
                tableName = tableAttrs[0].Name;
            }
            else
            {
                tableName = type.Name;
            }

            StringBuilder sql = new StringBuilder();
            sql.Append("UPDATE [").Append(tableName).Append("] SET ");
            int paramIndex = 0;

            for (int i = 0; i < properties.Length; i++)
            {
                var pi = properties[i];
                var idAttrs = (IdAttribute[])pi.GetCustomAttributes(typeof(IdAttribute), true);
                if (idAttrs.Length > 0) //如果是主键 跳过
                    continue;

                var columnAttrs = (ColumnAttribute[])pi.GetCustomAttributes(typeof(ColumnAttribute), true);
                if (columnAttrs.Length > 0)
                {
                    if (paramIndex > 0)
                        sql.Append(",");
                    // 字段 = @p
                    sql.Append(columnAttrs[0].Name).Append("=").Append("@p" + paramIndex);

                    /*参数*/
                    parameters.Add("@p" + paramIndex, pi.GetValue(entity, null));
                    paramIndex++;
                }
            }

            sql.Append(" WHERE ");
            int keyIndex = 0;
            for (int i = 0; i < properties.Length; i++)
            {
                var pi = properties[i];
                var idAttrs = (IdAttribute[])pi.GetCustomAttributes(typeof(IdAttribute), true);
                if (idAttrs.Length > 0)
                {
                    if (keyIndex > 0) //考虑到有多个主键
                        sql.Append(" AND ");
                    sql.Append(idAttrs[0].Name).Append("=").Append("@p").Append(paramIndex);

                    /*参数*/
                    parameters.Add("@p" + paramIndex, pi.GetValue(entity, null));
                    paramIndex++;
                    keyIndex++;
                }
            }
            Console.WriteLine(sql);

            SqlConnection conn = new SqlConnection(connectionString);
            var cmd = conn.CreateCommand();
            cmd.CommandText = sql.ToString();
            foreach (var item in parameters)
            {
                var pa = cmd.CreateParameter();
                pa.ParameterName = item.Key;
                pa.Value = item.Value ?? DBNull.Value;
                cmd.Parameters.Add(pa);
            }


            conn.Open();
            return cmd.ExecuteNonQuery();
        }

        public static int DeleteByKey<T>(params object[] values)
        {
            var type = typeof(T);
            Dictionary<string, object> parameters = new Dictionary<string, object>();
            var properties = type.GetProperties();

            string tableName = string.Empty;
            TableAttribute[] tableAttrs = (TableAttribute[])type.GetCustomAttributes(typeof(TableAttribute), true);
            if (tableAttrs.Length > 0)
            {
                tableName = tableAttrs[0].Name;
            }
            else
            {
                tableName = type.Name;
            }

            /*将所有的列放到集合里*/
            List<IdAttribute> columns = new List<IdAttribute>();
            for (int i = 0; i < properties.Length; i++)
            {
                var pi = properties[i];
                var attrs = (IdAttribute[])pi.GetCustomAttributes(typeof(IdAttribute), true);
                if (attrs.Length > 0)
                {
                    columns.Add(attrs[0]);
                }
            }
            if (columns.Count != values.Length)
                throw new ArgumentException("参数个数和主键数不一致");

            StringBuilder sql = new StringBuilder();
            sql.Append("DELETE FROM [").Append(tableName).Append("] ").Append(" WHERE ");

            for (int i = 0; i < columns.Count; i++)
            {
                if (i > 0) //考虑到有多个主键
                    sql.Append(" AND ");

                sql.Append(columns[i].Name).Append("=").Append("@p").Append(i);

                /*参数*/
                parameters.Add("@p" + i, values[i]);
            }
            Console.WriteLine(sql);

            SqlConnection conn = new SqlConnection(connectionString);
            var cmd = conn.CreateCommand();
            cmd.CommandText = sql.ToString();
            foreach (var item in parameters)
            {
                var pa = cmd.CreateParameter();
                pa.ParameterName = item.Key;
                pa.Value = item.Value ?? DBNull.Value;
                cmd.Parameters.Add(pa);
            }


            conn.Open();
            return cmd.ExecuteNonQuery();
        }

        public static T Get<T>(object[] values)
        {
            var type = typeof(T);
            Dictionary<string, object> parameters = new Dictionary<string, object>();
            var properties = type.GetProperties();

            string tableName = string.Empty;
            TableAttribute[] tableAttrs = (TableAttribute[])type.GetCustomAttributes(typeof(TableAttribute), true);
            if (tableAttrs.Length > 0)
            {
                tableName = tableAttrs[0].Name;
            }
            else
            {
                tableName = type.Name;
            }

            /*将所有的列放到集合里*/
            List<IdAttribute> columns = new List<IdAttribute>();
            for (int i = 0; i < properties.Length; i++)
            {
                var pi = properties[i];
                var attrs = (IdAttribute[])pi.GetCustomAttributes(typeof(IdAttribute), true);
                if (attrs.Length > 0)
                {
                    columns.Add(attrs[0]);
                }
            }
            if (columns.Count != values.Length)
                throw new ArgumentException("参数个数和主键数不一致");

            StringBuilder sql = new StringBuilder();
            sql.Append("SELECT * FROM [").Append(tableName).Append("] ").Append(" WHERE ");

            for (int i = 0; i < columns.Count; i++)
            {
                if (i > 0) //考虑到有多个主键
                    sql.Append(" AND ");

                sql.Append(columns[i].Name).Append("=").Append("@p").Append(i);

                /*参数*/
                parameters.Add("@p" + i, values[i]);
            }
            Console.WriteLine(sql);
            return GetEntityList<T>(sql.ToString(), parameters).FirstOrDefault();
        }
        public static List<T> GetList<T>(string where,string orderBy)
        {
            var type = typeof(T);
            Dictionary<string, object> parameters = new Dictionary<string, object>();
            var properties = type.GetProperties();

            string tableName = string.Empty;
            TableAttribute[] tableAttrs = (TableAttribute[])type.GetCustomAttributes(typeof(TableAttribute), true);
            if (tableAttrs.Length > 0)
            {
                tableName = tableAttrs[0].Name;
            }
            else
            {
                tableName = type.Name;
            }


            StringBuilder sql = new StringBuilder();
            sql.Append("SELECT * FROM [").Append(tableName).Append("] ");
            if (string.IsNullOrEmpty(where))
                sql.Append(" WHERE ").Append(where);

            if (string.IsNullOrEmpty(orderBy))
                sql.Append(" Order By ").Append(orderBy);


            Console.WriteLine(sql);
            return GetEntityList<T>(sql.ToString(), parameters);
        }

        public static List<T> GetEntityList<T>(string sql, Dictionary<string, object> parameters)
        {
            SqlConnection conn = new SqlConnection(EntityHelper.connectionString);
            conn.Open();

            var cmd = conn.CreateCommand();
            cmd.CommandText = sql;
            if (parameters != null && parameters.Count > 0)
            {
                foreach (var item in parameters)
                {
                    var pa = cmd.CreateParameter();
                    pa.ParameterName = item.Key;
                    pa.Value = item.Value ?? DBNull.Value;
                    cmd.Parameters.Add(pa);
                }
            }

            SqlDataReader reader = cmd.ExecuteReader();
            List<T> list = new List<T>();
            var type = typeof(T);
            var properties = type.GetProperties();
            while (reader.Read())
            {
                var user = Activator.CreateInstance(type);
                for (int i = 0; i < properties.Length; i++)
                {
                    var pi = properties[i];
                    if (reader[pi.Name] != null) //等同于 if (reader["UserId"] != null)这样的语句
                        pi.SetValue(user, reader[pi.Name], null); //等同于 user.UserId = (int)reader["UserId"];
                }

                list.Add((T)user);
            }
            return list;
        }
    }
}